@@ -15,17 +15,17 @@
 # limitations under the License.
 
 import sys
-import zmq
 
-from networking_mlnx._i18n import _, _LE, _LI
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+import zmq
 
+from networking_mlnx._i18n import _
 from networking_mlnx.eswitchd.common import config
 from networking_mlnx.eswitchd.common import constants
 from networking_mlnx.eswitchd.eswitch_handler import eSwitchHandler
-import networking_mlnx.eswitchd.msg_handler as message
+from networking_mlnx.eswitchd import msg_handler as message
 from networking_mlnx.eswitchd.utils.helper_utils import set_conn_url
 
 LOG = logging.getLogger(__name__)
@@ -51,13 +51,12 @@ class MlxEswitchDaemon(object):
                     fabric, pf = entry.split(':')
                     fabrics.append((fabric, pf))
                 except ValueError:
-                    LOG.error(_LE("Invalid fabric: "
-                                "'%(entry)s' - "
-                                "Service terminated!"),
-                              locals())
+                    LOG.error("Invalid fabric: "
+                              "'%(entry)s' - "
+                              "Service terminated!", locals())
                     raise
             else:
-                LOG.error(_LE("Cannot parse Fabric Mappings"))
+                LOG.error("Cannot parse Fabric Mappings")
                 raise Exception(_("Cannot parse Fabric Mappings"))
         return fabrics
 
@@ -87,12 +86,12 @@ class MlxEswitchDaemon(object):
                 result = self.dispatcher.handle_msg(data)
                 msg = jsonutils.dumps(result)
             except Exception as e:
-                LOG.exception(_LE("Exception during message handling - %s"), e)
+                LOG.exception("Exception during message handling - %s", e)
                 msg = str(e)
-            sender.send(msg)
+            sender.send_string(msg)
 
     def daemon_loop(self):
-        LOG.info(_LI("Daemon Started!"))
+        LOG.info("Daemon Started!")
         polling_counter = 0
         while True:
             self._handle_msg()
@@ -106,13 +105,12 @@ class MlxEswitchDaemon(object):
 
 def main():
     config.init(sys.argv[1:])
-    config.setup_logging()
     try:
         daemon = MlxEswitchDaemon()
         daemon.start()
     except Exception as e:
-        LOG.exception(_LE("Failed to start EswitchDaemon "
-                          "- Daemon terminated! %s"), e)
+        LOG.exception("Failed to start EswitchDaemon "
+                      "- Daemon terminated! %s", e)
         sys.exit(1)
 
     daemon.daemon_loop()
